% Results
%close all; 
clear; clc;

% Solution options
lang = 'M';
runstr = 'dens'; 
mod = 'base';

% Simulation options
%   Long simulation
nburn = 1e3;
nperlong = 1e4;
nshocks = 3;
ps = [0.005,0.995];
%   Decomposition
nh = 1;
nsimsTVD = 5000;
%   Short simulations
npershort = nh+1;
F.nlagQ = 1;
F.ndataQ = 200;
F.nburn = 1e3;
F.nreps = 5001;

% Set random number seed and draw shocks
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);
shockslong = randn(nburn+nperlong,1,nshocks);
shocksshort = randn(F.nburn+F.ndataQ,F.nreps,nshocks);
shocksTVD = randn(npershort,nsimsTVD);

% Load solution
fname = ['mats/' lang 'solution-' runstr '-' mod '.mat'];
disp(fname); load(fname)

% Simulation results
disp('    Long simulations')
sims = simulation(pf,P,S,G,V,shockslong);
sims = sims(nburn+1:end,:);
ergmean = mean(sims,1)';
disater_per = 100*nnz(sims(:,V.Theta)<1)/nperlong;
disp(['    disater_per = ' num2str(disater_per)])

%       Disaster risk volatility 
boundsper = G.lnsige_grid([1,end]);
medpsper = quantile(sims(:,V.sige),ps);
perout = sims(:,V.sige) <= min(G.lnsige_grid) | sims(:,V.sige) >= max(G.lnsige_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       sige is out of bounds')
end
disp(['     sige bounds:  ',num2str(boundsper)])
disp(['     sige ergodic: ',num2str(medpsper)])
disp(['     sige % out:   ',num2str(100*mean(perout))])
%       Disaster risk level
boundsper = G.lne_grid([1,end]);
medpsper = quantile(sims(:,V.e),ps);
perout = sims(:,V.e) <= min(G.lne_grid) | sims(:,V.e) >= max(G.lne_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       e is out of bounds')
end
disp(['     e bounds:  ',num2str(boundsper)])
disp(['     e ergodic: ',num2str(medpsper)])
disp(['     e % out:   ',num2str(100*mean(perout))])
%       TFP shock
boundsper = G.lna_grid([1,end]);
medpsper = quantile(sims(:,V.a),ps);
perout = sims(:,V.a) <= min(G.lna_grid) | sims(:,V.a) >= max(G.lna_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
    warning('       a is out of bounds')
end
disp(['     a bounds:  ',num2str(boundsper)])
disp(['     a ergodic: ',num2str(medpsper)])
disp(['     a % out:   ',num2str(100*mean(perout))])
%	Capital
boundsper = G.k_grid([1,end])./S.k-1;
medpsper = quantile(sims(:,V.k),ps)./S.k-1;
perout = sims(:,V.k) <= min(G.k_grid) | sims(:,V.k) >= max(G.k_grid);
if medpsper(1) < boundsper(1) || medpsper(2) > boundsper(2)
	warning('       k is out of bounds')
end
disp(['     k bounds:  ',num2str(boundsper)])
disp(['     k ergodic: ',num2str(medpsper)])
disp(['     k % out:   ',num2str(100*mean(perout))])
disp('')

%% Compute short-sample moments
disp('	Short-sample simulations...');
sims = simulation(pf,P,S,G,V,shocksshort);
sims = sims(F.nburn+1:end,:,:);
M = momcalc(V,F,sims);

% Load data moments
load('../../Data/moments.mat')

% Display key moments 
mom_varidx = [V.y,V.uy];
mom_cstrings = {'Data','Model'};
mom_rstrings = {'SD(y)';'SD(a)';'AC(y)';'AC(a)'};
dsge_moms = [M.sd_yf,M.sd_af,M.ac_yf,M.ac_af]';
tabmoms = array2table([data_moms,dsge_moms]);
tabmoms = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tabmoms);
tabmoms.Properties.VariableNames = mom_cstrings;
tabmoms.Properties.RowNames = mom_rstrings;
disp(tabmoms)

%% Total variance decomposition
disp('	TVD'); 
Eysim_exevol = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exevol = NaN(nh,V.nplotvar,nsimsTVD);
Eysim_exalev = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exalev = NaN(nh,V.nplotvar,nsimsTVD);
Eysim_exelev = NaN(nh,V.nplotvar,nsimsTVD);
Vysim_exelev = NaN(nh,V.nplotvar,nsimsTVD);
for isim = 1:nsimsTVD
    % Simulate
    %   Fixed shocks to disaster risk volatility
    shocks = cat(3,repmat(shocksTVD(:,isim),[1,nsimsTVD]),shocksTVD,shocksTVD);
    ysim_Fevol = simulation(pf,P,S,G,V,shocks,ergmean);
    %   Fixed shocks to disaster risk level
    shocks = cat(3,shocksTVD,repmat(shocksTVD(:,isim),[1,nsimsTVD]),shocksTVD);
    ysim_Felev = simulation(pf,P,S,G,V,shocks,ergmean);
    %   Fixed shocks to TFP level
    shocks = cat(3,shocksTVD,shocksTVD,repmat(shocksTVD(:,isim),[1,nsimsTVD]));
    ysim_Falev = simulation(pf,P,S,G,V,shocks,ergmean);
    % Integrate
    %   Across all shocks, except disaster risk volatility
    Eysim_exevol(:,:,isim) = mean(ysim_Fevol(2:end,:,:),3,'omitnan');
    Vysim_exevol(:,:,isim) = var(ysim_Fevol(2:end,:,:),[],3,'omitnan');
    %   Across all shocks, except disaster risk level
    Eysim_exelev(:,:,isim) = mean(ysim_Felev(2:end,:,:),3,'omitnan');
    Vysim_exelev(:,:,isim) = var(ysim_Felev(2:end,:,:),[],3,'omitnan');
    %   Across all shocks, except TFP level
    Eysim_exalev(:,:,isim) = mean(ysim_Falev(2:end,:,:),3,'omitnan');
    Vysim_exalev(:,:,isim) = var(ysim_Falev(2:end,:,:),[],3,'omitnan');
end
% Integrate
%   Across TFP level shocks
TE_exalev = mean(Vysim_exalev,3)';
DE_alev = var(Eysim_exalev,[],3)';
%   Across disaster risk level shocks
TE_exelev = mean(Vysim_exelev,3)';
DE_elev = var(Eysim_exelev,[],3)';
%   Across disaster volatility shocks
TE_exevol = mean(Vysim_exevol,3)';
DE_evol = var(Eysim_exevol,[],3)';
% Direct Effect Variance Contribution
evol_Direct = 100*DE_evol./(TE_exevol+DE_evol);
elev_Direct = 100*DE_elev./(TE_exelev+DE_elev);
alev_Direct = 100*DE_alev./(TE_exalev+DE_alev);

% Display TVD results 
tvd_varidx = [V.y,V.uy,V.i];
tvd_cstrings = {'Output','Uncertainty','Investment'};
tvd_rstrings = {'TFP Level Shock','DR Volatility Shock','DR Level Shock'};
tab_out = [alev_Direct(tvd_varidx,nh)';
           evol_Direct(tvd_varidx,nh)';
           elev_Direct(tvd_varidx,nh)'];
tabvardecomp = array2table(tab_out);
tabvardecomp = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tabvardecomp);
tabvardecomp.Properties.VariableNames = tvd_cstrings;
tabvardecomp.Properties.RowNames = tvd_rstrings;
disp(tabvardecomp)