% Simulate model and estimate VAR
clear; clc; close all;
warning('off','backtrace')

%Saving
saving = 1;

% VAR options
O.mod = 'qr';       % Baseline: base; Quasi-recursive: qr
Ovar.cholflag = 1;    % Cholesky = 1
K = 3;              % Number of variables
lags = 4;           % Set number of lags
nlags = numel(lags);

noises = [0,0.2,0.5,1];
nnoises = numel(noises);

% VAR order
V = variables;
firstvar = 'sigenoise';
secondvar = 'uy';
if K ==3
    lastvar = 'y';  %output: y; consumption: c; investment: i; TFP: a;
end
varidx = {firstvar,secondvar,lastvar};
Ovar.ivarnews = find(strcmp(varidx,'sigenoise'));
Ovar.ivaruy = find(strcmp(varidx,'uy'));
Ovar.ivary = find(strcmp(varidx,'y'));

% Max share method (if applicable)
if Ovar.cholflag == 1
    Ovar.carrieroflag = NaN; 
elseif Ovar.cholflag ==0
    % Carriero and Volpicella = 1 
    % Standard max share = 0
    Ovar.carrieroflag = 0;
end
if Ovar.carrieroflag ==0
    Ovar.ntarget = 1; %Max-share on first variable
else
    Ovar.ntarget = 3; %Set 2 to K
end
if Ovar.ntarget>K
    disp('# of identified shocks cannot be > # of variables in VAR');
    return
end

% Set target horizons. Can be set different for K > 1
if Ovar.ntarget == 1
    Ovar.hvar = 0;
elseif Ovar.ntarget == 2
    Ovar.hvar = [0;0]; 
elseif Ovar.ntarget == 3
    Ovar.hvar = [0;0;0];
end
Ovar.hirf = 81;

% FEVD horizons
Ovar.hvars_fevd = [4,20,40,80];
Ovar.nhvars_fevd = numel(Ovar.hvars_fevd);
for ihvar=1:Ovar.nhvars_fevd
    colstrs_fevd{ihvar} = ['h=' num2str(Ovar.hvars_fevd(ihvar))];
end

% Load simulations
load(['mats/simdata-' O.mod '.mat']);
sims(sims(:,V.uy)<0,V.uy)=0.0001;
ievolidx = 1;
ielevidx = 2;
ialevidx = 3;

% VAR
disp('Estimate VAR...');
%   Dimension: Horizons, Variables, Shocks, Reps, Lags
irfs = NaN(Ovar.hirf+1,K,K,F.nreps,nlags,nnoises);
%   Dimension: Shocks, Variables, Horizons, Reps
fevds = NaN(K,K,Ovar.nhvars_fevd,F.nreps);

for inoise = 1:nnoises
    mtstream = RandStream('mt19937ar','seed',1);
    RandStream.setGlobalStream(mtstream);
    
    %Generate noise shock
    P = parameters(O);
    P.signoise = noises(inoise)*P.sigsv;
    epsnoise = P.signoise*randn(1,size(sims,1)-1,F.nreps);
    
    for ilag = 1:nlags
        lag = lags(ilag);
        Ovar.lag = lag;
        disp(['Order: ',varidx{1},', ',varidx{2},', lag' num2str(lag)])    
        corrarray = zeros(K*3,F.nreps);    
        for irep = 1:F.nreps 
            % News
            simsige = 100*sims(2:end,V.sige,irep);
            simsigenoise = (simsige/100) + epsnoise';
            simuy = 100*log(sims(2:end,V.uy,irep)); 
            simy = 100*log(sims(2:end,V.y,irep));
            sima =  100*sims(2:end,V.a,irep);        
            if strcmp(secondvar,'y')
                simsecond = simy;
            elseif strcmp(secondvar,'uy')
                simsecond = simuy;
            end
            if strcmp(lastvar,'y')
                simlast = simy;
            elseif strcmp(lastvar,'a')
                simlast = sima;
            end

            % Assign data in VAR order
            if K ==2 
                vardata = [simsigenoise,simsecond];
            elseif K ==3
                vardata = [simsigenoise,simsecond,simlast];
            end

            % Estimate
            [irfs(:,:,:,irep,ilag,inoise),x,struct_shocks_var,fevds(:,:,:,irep)] = ...
                var_maxshare(vardata,F,V,Ovar);

            % Correlations of DSGE structural shocks and Cholesky identified shocks 
            struct_shocks_dsge = shockslong((F.nburn+1+1+lag):(F.nburn+F.ndata),:,irep);
            %   e level shock v. VAR news shock
            corrarray(1,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivarnews));
            %   e vol shock v. VAR news shock
            corrarray(2,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivarnews));
            %   a level shock v. VAR news shock
            corrarray(3,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivarnews));
            %   e level shock v. VAR uncertainty shock
            corrarray(4,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivaruy));
            %   e vol shocl v. VAR uncertainty shock
            corrarray(5,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivaruy));
            %  a level shock v. VAR uncertainty shock
            corrarray(6,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivaruy));
            if K ==3
                %   e level shock v. VAR output shock
                corrarray(7,irep) = corr(struct_shocks_dsge(:,ielevidx), struct_shocks_var(:,Ovar.ivary));
                %   e vol shock v. VAR output shock
                corrarray(8,irep) = corr(struct_shocks_dsge(:,ievolidx), struct_shocks_var(:,Ovar.ivary));
                %   a level shock v. VAR output shock
                corrarray(9,irep) = corr(struct_shocks_dsge(:,ialevidx), struct_shocks_var(:,Ovar.ivary));
            end
        end           

        % Shock correlations table
        corrarray_mean = mean(corrarray,2);
        tab = array2table(corrarray_mean);
        tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',2)]), tab);
        if K ==2 
            tab.Properties.RowNames = {...
            'Corr(e lev shock, VAR sig e shock)',...
            'Corr(e vol shock, VAR sig e shock)',...
            'Corr(a lev shock, VAR sig e shock)',...
            'Corr(e lev shock, VAR second shock)',...
            'Corr(e vol shock, VAR second shock)',...
            'Corr(a lev shock, VAR second shock)'};
        elseif K ==3
             tab.Properties.RowNames = {...
            'Corr(e lev shock, VAR sig e shock)',...
            'Corr(e vol shock, VAR sig e shock)',...
            'Corr(a lev shock, VAR sig e shock)',...
            'Corr(e lev shock, VAR second shock)',...
            'Corr(e vol shock, VAR second shock)',...
            'Corr(a lev shock, VAR second shock)',...
            'Corr(e lev shock, VAR last shock)',...
            'Corr(e vol shock, VAR last shock)',...
            'Corr(a lev shock, VAR last shock)'};
        end
        tab.Properties.VariableNames = {'Mean'};
        disp(tab)

        % FEVD table
        % fevd_mean = 100*squeeze(mean(fevds,4));
        % tab = array2table(fevd_mean(ivary,:)');
        % tab = varfun(@(x) num2str(x, ['%',sprintf('.%df',1)]), tab);
        % tab.Properties.RowNames = colstrs;
        % tab.Properties.VariableNames = {'Output FEVD'};
        % disp(tab) 
    end   
    if saving==1
        if Ovar.cholflag
            if K ==2
                filename = ['mats/var-' firstvar '-' secondvar ...
                    '-' O.mod '-chol-news-noise.mat'];
            elseif K ==3
                filename = ['mats/var-' firstvar '-' secondvar '-' lastvar ...
                    '-' O.mod '-chol-news-noise.mat'];
            end
        else
            if K ==2
                filename = ['mats/var-' firstvar '-' secondvar ...
                    '-' O.mod '-ntarget' num2str(Ovar.ntarget) ...
                    '-hvar' num2str(Ovar.hvar(1)) ...
                    '-carr' num2str(Ovar.carrieroflag) '-news-noise.mat'];
            elseif K ==3
                filename = ['mats/var-' firstvar '-' secondvar ...
                    '-' lastvar '-' O.mod '-ntarget' num2str(Ovar.ntarget) ...
                    '-hvar' num2str(Ovar.hvar(1)) ...
                    '-carr' num2str(Ovar.carrieroflag) '-news-noise.mat'];
            end
        end    
        save(filename,'irfs','varidx','lags','ievolidx','noises');
    end
end